@@ -84,6 +84,11 @@ def print_error(
     )
 
 
+def show_spinning_status(text: str):
+    """Returns a context manager."""
+    return get_console().status(text)
+
+
 def get_model_title(model_cls: t.Type[BaseModel]) -> str:
     if model_cls.__config__.title:
         return model_cls.__config__.title
@@ -103,7 +108,8 @@ def print_model_field_values(
 ):
     for k, v in port_values.items():
         rprint(f"\n{icon if icon else '***'} {k}:")
-        if model_fields[k].field_info.description:
+        # Ports might be virtual, as in ShellCommand, so they might not be in the model
+        if k in model_fields and model_fields[k].field_info.description:
             rprint(
                 f"[italic grey50]{escape(model_fields[k].field_info.description)}[/]"
             )
@@ -187,11 +193,23 @@ def _field_row(
     is_model = _is_model(field.outer_type_) and not inspect.isabstract(
         field.outer_type_
     )
-    has_root_item = ("__root__" in field.outer_type_.__fields__) if is_model else False
-    field_outer_type = field.outer_type_
 
-    if has_root_item:
-        field_outer_type = field.outer_type_.__fields__["__root__"].outer_type_
+    # NB: docs should not come from the inner __root__ type
+    if field.field_info.description:
+        field_docs = field.field_info.description
+    elif hasattr(field.outer_type_, "__doc__") and field.outer_type_.__doc__:
+        field_docs = field.outer_type_.__doc__
+    else:
+        field_docs = ""
+
+    field_docs = " ".join(field_docs.split())
+
+    has_root_item = ("__root__" in field.outer_type_.__fields__) if is_model else False
+    field_outer_type = (
+        field.outer_type_.__fields__["__root__"].outer_type_
+        if has_root_item
+        else field.outer_type_
+    )
 
     if show_piper_port:
         from pipelime.piper import PiperPortType
@@ -211,6 +229,7 @@ def _field_row(
 
     line = (
         [
+            # Field name & alias
             (" " * indent)
             + ("[bold salmon1]" if indent == 0 else "")
             + (
@@ -220,16 +239,20 @@ def _field_row(
             )
             + f"{escape(field.alias)}"
             + ("[/]" if indent == 0 else ""),
-            ("▶ " + escape(field.field_info.description))
-            if field.field_info.description
-            else "",
+            # Description
+            ("▶ " + escape(field_docs)),
+            # Type
             (
                 ""
                 if is_model and not has_root_item
-                else display_as_type(field_outer_type).replace("[", r"\[")  # noqa: W605
+                else _human_readable_type(field_outer_type).replace(
+                    "[", r"\["
+                )  # noqa: W605
             ),
         ]
+        # Piper port
         + ([fport] if fport else [])
+        # Default value
         + (["[red]✗[/]"] if field.required else [f"[green]{field.get_default()}[/]"])
     )
 
@@ -255,7 +278,10 @@ def _field_row(
         }
 
         for arg in inner_types:
-            grid.add_row((" " * indent) + f"[grey50]{_short_line()} {arg.__name__}[/]")
+            grid.add_row(
+                (" " * indent)
+                + f"[grey50]{_short_line()} {arg.__name__}[/]"  # type:ignore
+            )
             _iterate_model_fields(
                 model_cls=arg,
                 grid=grid,
@@ -284,6 +310,15 @@ def _is_model(type_):
     return inspect.isclass(type_) and issubclass(type_, BaseModel)
 
 
+def _human_readable_type(field_outer_type):
+    from enum import Enum
+
+    tstr = display_as_type(field_outer_type)
+    if inspect.isclass(field_outer_type) and issubclass(field_outer_type, Enum):
+        tstr += " {" + ", ".join(v.name.lower() for v in field_outer_type) + "}"
+    return tstr
+
+
 def _recursive_args_flattening(arg):
     if isinstance(arg, t.Mapping):
         return {
@@ -292,7 +327,7 @@ def _recursive_args_flattening(arg):
             for u in (_recursive_args_flattening(k), _recursive_args_flattening(v))
             for a in u
         }
-    if isinstance(arg, t.Collection):
+    if isinstance(arg, t.Collection) and not isinstance(arg, (str, bytes)):
         return {a for v in arg for a in _recursive_args_flattening(v)}
     return {arg}
 
